@@ -13,10 +13,16 @@ aum_line_search <- structure(function
   pred.vec=NULL,
 ### N-vector of numeric predicted values. If NULL, feature.mat and
 ### weight.vec will be used to compute predicted values.
-  maxIterations=nrow(error.diff.df)
+  maxIterations=nrow(error.diff.df),
 ### max number of line search iterations, either a positive integer or
 ### "max.auc" or "min.aum" indicating to keep going until AUC
 ### decreases or AUM increases.
+  feature.mat.search=feature.mat,
+### feature matrix to use in line search, default is subtrain, can be validation
+  error.diff.search=error.diff.df,
+### aum_diffs data frame to use in line search, default is subtrain, can be validation
+  maxStepSize=-1
+### max step size to explore.
 ){
   . <- fp.diff <- fn.diff <- intercept <- slope <- step.size <- NULL
   ## Above to suppress CRAN NOTE.
@@ -26,18 +32,23 @@ aum_line_search <- structure(function
   }
   L <- aum(error.diff.df, pred.vec)
   L$pred.vec <- pred.vec
+  L$pred.vec.search <- if(pred.null){
+    feature.mat.search %*% weight.vec
+  }else{
+    pred.vec
+  }
   L$gradient_pred <- rowMeans(L$derivative_mat)
   L$gradient <- if(pred.null){
     L$gradient_weight <- t(feature.mat) %*% L$gradient_pred
-    feature.mat %*% L$gradient_weight
+    feature.mat.search %*% L$gradient_weight
   }else{
     L$gradient_pred
   }
-  pred.i <- error.diff.df$example+1L
+  pred.i <- error.diff.search$example+1L
   L$line_search_input <- data.table(
-    fp.diff=error.diff.df$fp_diff,
-    fn.diff=error.diff.df$fn_diff,
-    intercept=error.diff.df$pred-pred.vec[pred.i],
+    fp.diff=error.diff.search$fp_diff,
+    fn.diff=error.diff.search$fn_diff,
+    intercept=error.diff.search$pred-L$pred.vec.search[pred.i],
     slope=L$gradient[pred.i]
   )[, .(
     fp.diff=sum(fp.diff),
@@ -45,7 +56,7 @@ aum_line_search <- structure(function
   ), keyby=.(intercept, slope)]
   if(identical(maxIterations, "max.auc"))maxIterations <- -1L
   if(identical(maxIterations, "min.aum"))maxIterations <- 0L
-  line.search.all <- aumLineSearch(L$line_search_input, maxIterations)
+  line.search.all <- aumLineSearch(L$line_search_input, maxIterations, maxStepSize)
   L$line_search_result <- data.table(line.search.all)[0 <= step.size]
   class(L) <- c("aum_line_search", class(L))
   L
@@ -65,53 +76,135 @@ aum_line_search <- structure(function
   bin.line.search <- aum::aum_line_search(bin.diffs, pred.vec=c(10,-10))
   if(requireNamespace("ggplot2"))plot(bin.line.search)
 
-  ## Example 2: two changepoint examples, one with three breakpoints.
-  data(neuroblastomaProcessed, package="penaltyLearning", envir=environment())
-  nb.err <- with(neuroblastomaProcessed$errors, data.frame(
-    example=paste0(profile.id, ".", chromosome),
-    min.lambda,
-    max.lambda,
-    fp, fn))
-  (nb.diffs <- aum::aum_diffs_penalty(nb.err, c("1.1", "4.2")))
-  if(requireNamespace("ggplot2"))plot(nb.diffs)
-  nb.line.search <- aum::aum_line_search(nb.diffs, pred.vec=c(1,-1))
-  if(requireNamespace("ggplot2"))plot(nb.line.search)
-  aum::aum_line_search(nb.diffs, pred.vec=c(1,-1)-c(1,-1)*0.5)
+  if(requireNamespace("penaltyLearning")){
 
-  ## Example 3: all changepoint examples, with linear model.
-  X.sc <- scale(neuroblastomaProcessed$feature.mat)
-  keep <- apply(is.finite(X.sc), 2, all)
-  X.keep <- X.sc[1:50,keep]
-  weight.vec <- rep(0, ncol(X.keep))
-  (nb.diffs <- aum::aum_diffs_penalty(nb.err, rownames(X.keep)))
-  nb.weight.search <- aum::aum_line_search(
-    nb.diffs,
-    feature.mat=X.keep,
-    weight.vec=weight.vec, 
-    maxIterations = 200)
-  if(requireNamespace("ggplot2"))plot(nb.weight.search)
+    ## Example 2: two changepoint examples, one with three breakpoints.
+    data(neuroblastomaProcessed, package="penaltyLearning", envir=environment())
+    nb.err <- with(neuroblastomaProcessed$errors, data.frame(
+      example=paste0(profile.id, ".", chromosome),
+      min.lambda,
+      max.lambda,
+      fp, fn))
+    (nb.diffs <- aum::aum_diffs_penalty(nb.err, c("1.1", "4.2")))
+    if(requireNamespace("ggplot2"))plot(nb.diffs)
+    nb.line.search <- aum::aum_line_search(nb.diffs, pred.vec=c(1,-1))
+    if(requireNamespace("ggplot2"))plot(nb.line.search)
+    aum::aum_line_search(nb.diffs, pred.vec=c(1,-1)-c(1,-1)*0.5)
 
-  ## Alternate viz with x=iteration instead of step size.
-  nb.weight.full <- aum::aum_line_search(
-    nb.diffs,
-    feature.mat=X.keep,
-    weight.vec=weight.vec, 
-    maxIterations = 1000)
-  library(data.table)
-  weight.result.tall <- suppressWarnings(melt(
-    nb.weight.full$line_search_result[, iteration:=1:.N][, .(
-      iteration, auc, q.size,
-      log10.step.size=log10(step.size),
-      log10.aum=log10(aum))],
-    id.vars="iteration"))
-  if(require(ggplot2)){
+    ## Example 3: all changepoint examples, with linear model.
+    X.sc <- scale(neuroblastomaProcessed$feature.mat)
+    keep <- apply(is.finite(X.sc), 2, all)
+    X.subtrain <- X.sc[1:50,keep]
+    weight.vec <- rep(0, ncol(X.subtrain))
+    (diffs.subtrain <- aum::aum_diffs_penalty(nb.err, rownames(X.subtrain)))
+    nb.weight.search <- aum::aum_line_search(
+      diffs.subtrain,
+      feature.mat=X.subtrain,
+      weight.vec=weight.vec, 
+      maxIterations = 200)
+    if(requireNamespace("ggplot2"))plot(nb.weight.search)
+
+    ## Stop line search after finding a (local) max AUC or min AUM.
+    max.auc.search <- aum::aum_line_search(
+      diffs.subtrain,
+      feature.mat=X.subtrain,
+      weight.vec=weight.vec,
+      maxIterations="max.auc")
+    min.aum.search <- aum::aum_line_search(
+      diffs.subtrain,
+      feature.mat=X.subtrain,
+      weight.vec=weight.vec,
+      maxIterations="min.aum")
+    if(require("ggplot2")){
+      plot(nb.weight.search)+
+        geom_point(aes(
+          step.size, auc),
+          data=data.table(max.auc.search[["line_search_result"]], panel="auc"),
+          color="red")+
+        geom_point(aes(
+          step.size, aum),
+          data=data.table(min.aum.search[["line_search_result"]], panel="aum"),
+          color="red")
+    }
+
+    ## Alternate viz with x=iteration instead of step size.
+    nb.weight.full <- aum::aum_line_search(
+      diffs.subtrain,
+      feature.mat=X.subtrain,
+      weight.vec=weight.vec, 
+      maxIterations = 1000)
+    library(data.table)
+    weight.result.tall <- suppressWarnings(melt(
+      nb.weight.full$line_search_result[, iteration:=1:.N][, .(
+        iteration, auc, q.size,
+        log10.step.size=log10(step.size),
+        log10.aum=log10(aum))],
+      id.vars="iteration"))
+    if(require(ggplot2)){
+      ggplot()+
+        geom_point(aes(
+          iteration, value),
+          shape=1,
+          data=weight.result.tall)+
+        facet_grid(variable ~ ., scales="free")+
+        scale_y_continuous("")
+    }
+
+    ## Example 4: line search on validation set.
+    X.validation <- X.sc[101:300,keep]
+    diffs.validation <- aum::aum_diffs_penalty(nb.err, rownames(X.validation))
+    valid.search <- aum::aum_line_search(
+      diffs.subtrain,
+      feature.mat=X.subtrain,
+      weight.vec=weight.vec, 
+      maxIterations = 2000,
+      feature.mat.search=X.validation,
+      error.diff.search=diffs.validation)
+    if(requireNamespace("ggplot2"))plot(valid.search)
+
+    ## validation set max auc, min aum.
+    max.auc.valid <- aum::aum_line_search(
+      diffs.subtrain,
+      feature.mat=X.subtrain,
+      weight.vec=weight.vec,
+      maxIterations="max.auc",
+      feature.mat.search=X.validation,
+      error.diff.search=diffs.validation)
+    min.aum.valid <- aum::aum_line_search(
+      diffs.subtrain,
+      feature.mat=X.subtrain,
+      weight.vec=weight.vec,
+      maxIterations="min.aum",
+      feature.mat.search=X.validation,
+      error.diff.search=diffs.validation)
+    if(require("ggplot2")){
+      plot(valid.search)+
+        geom_point(aes(
+          step.size, auc),
+          data=data.table(max.auc.valid[["line_search_result"]], panel="auc"),
+          color="red")+
+        geom_point(aes(
+          step.size, aum),
+          data=data.table(min.aum.valid[["line_search_result"]], panel="aum"),
+          color="red")
+    }
+
+    ## compare subtrain and validation
+    both.results <- rbind(
+      data.table(valid.search$line_search_result, set="validation"),
+      data.table(nb.weight.search$line_search_result, set="subtrain"))
+    both.max <- rbind(
+      data.table(max.auc.valid$line_search_result, set="validation"),
+      data.table(max.auc.search$line_search_result, set="subtrain"))
     ggplot()+
+      geom_vline(aes(
+        xintercept=step.size, color=set),
+        data=both.max)+
       geom_point(aes(
-        iteration, value),
+        step.size, auc, color=set),
         shape=1,
-        data=weight.result.tall)+
-      facet_grid(variable ~ ., scales="free")+
-      scale_y_continuous("")
+        data=both.results)
+
   }
   
 })
@@ -252,30 +345,34 @@ aum_line_search_grid <- structure(function
   bin.line.search <- aum::aum_line_search_grid(bin.diffs, pred.vec=c(-10,10))
   if(requireNamespace("ggplot2"))plot(bin.line.search)
 
-  ## Example 2: two changepoint examples, one with three breakpoints.
-  data(neuroblastomaProcessed, package="penaltyLearning", envir=environment())
-  nb.err <- with(neuroblastomaProcessed$errors, data.frame(
-    example=paste0(profile.id, ".", chromosome),
-    min.lambda,
-    max.lambda,
-    fp, fn))
-  (nb.diffs <- aum::aum_diffs_penalty(nb.err, c("4.2", "1.1")))
-  if(requireNamespace("ggplot2"))plot(nb.diffs)
-  (nb.line.search <- aum::aum_line_search_grid(nb.diffs, pred.vec=c(-1,1)))
-  if(requireNamespace("ggplot2"))plot(nb.line.search)
+  if(requireNamespace("penaltyLearning")){
 
-  ## Example 3: 50 changepoint examples, with linear model.
-  X.sc <- scale(neuroblastomaProcessed$feature.mat[1:50,])
-  keep <- apply(is.finite(X.sc), 2, all)
-  X.keep <- X.sc[,keep]
-  weight.vec <- rep(0, ncol(X.keep))
-  nb.diffs <- aum::aum_diffs_penalty(nb.err, rownames(X.keep))
-  nb.weight.search <- aum::aum_line_search_grid(
-    nb.diffs,
-    feature.mat=X.keep,
-    weight.vec=weight.vec,
-    maxIterations = 200)
-  if(requireNamespace("ggplot2"))plot(nb.weight.search)
+    ## Example 2: two changepoint examples, one with three breakpoints.
+    data(neuroblastomaProcessed, package="penaltyLearning", envir=environment())
+    nb.err <- with(neuroblastomaProcessed$errors, data.frame(
+      example=paste0(profile.id, ".", chromosome),
+      min.lambda,
+      max.lambda,
+      fp, fn))
+    (diffs.subtrain <- aum::aum_diffs_penalty(nb.err, c("4.2", "1.1")))
+    if(requireNamespace("ggplot2"))plot(diffs.subtrain)
+    (nb.line.search <- aum::aum_line_search_grid(diffs.subtrain, pred.vec=c(-1,1)))
+    if(requireNamespace("ggplot2"))plot(nb.line.search)
+
+    ## Example 3: 50 changepoint examples, with linear model.
+    X.sc <- scale(neuroblastomaProcessed$feature.mat[1:50,])
+    keep <- apply(is.finite(X.sc), 2, all)
+    X.subtrain <- X.sc[,keep]
+    weight.vec <- rep(0, ncol(X.subtrain))
+    diffs.subtrain <- aum::aum_diffs_penalty(nb.err, rownames(X.subtrain))
+    nb.weight.search <- aum::aum_line_search_grid(
+      diffs.subtrain,
+      feature.mat=X.subtrain,
+      weight.vec=weight.vec,
+      maxIterations = 200)
+    if(requireNamespace("ggplot2"))plot(nb.weight.search)
+
+  }
 
   ## Example 4: counting intersections and intervals at each
   ## iteration/step size, when there are ties.
